#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2022
#
#   History:
#
#   v1.0: 2022-03: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
#FixWH=2000   # now dinamic

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer, WrkWH):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (WrkWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # take care of the existing mean in determining the theoretical mean
    meanadj=25
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    #info_msg ("values mean="+str(meanl))
    qobj = 127.5*meanadj/100
    qsrc = meanl*(100.0-meanadj)/100
    thmean = qobj+qsrc   
    #info_msg ("q_obj,q_src,target="+str(qobj)+","+str(qsrc)+","+str(thmean))
    # range for the loops
    minmean = thmean*1.01  
    maxmean = thmean*0.99  
    
    modified=False
    if meanl < minmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl)+" ratio = "+str(ratio))
        loop=0
        while meanl < minmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        modified=True
        #info_msg("increased mean value = "+str(meanl))
    if meanl > maxmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl))
        loop=0
        while meanl > maxmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        modified=True
        #info_msg("reduced mean value = "+str(meanl))

    return(inLayer)

############################################################################
# Main
#---------------------------------------------------------------------------

def GrainsOfColours (inImage, inLayer,
                  UserOutWH, EffectLook, AdjLayerMode, SourceOnTop,
                  doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    WrkWH=1500+1500*int(EffectLook)
    newImage = createWrkimage (inImage, inLayer, WrkWH)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Apply a Paint look
    # thru Segmentation  
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -fx_segment_watershed 12,4,0,0,0,50,50 ')
    Painted = newImage.layers[0]
    Painted.mode = LAYER_MODE_NORMAL
    Painted.opacity = 100.0
    Painted.name = "Preprocessed painted thru Segmentation"
    #pdb.plug_in_unsharp_mask (newImage, Painted, 3.0, 3.0, 0)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Apply Couleurs_Metalliques 0,0,100,0,0,1,60,6,5,20,1,0,0,0,0.3,60,1,1,0
    Metalliquesparms = '0,0,100,0,0,1,60,6,5,20,1,0,0,0,0.3,60,1,1,0'
    # to avoid halo try 0,0,100,0,0,1,2,2,5,20,1,0,0,0,0.3,60,1,1,0
    Metalliquesparms = '0,0,100,0,0,1,2,2,5,20,1,0,0,0,0.3,60,1,1,0'
    pdb.plug_in_gmic_qt(newImage, Painted, 1,2,
    '-v - -Couleurs_Metalliques '+Metalliquesparms+' ')
    Metalliques = newImage.layers[0]
    Metalliques.mode = LAYER_MODE_GRAIN_MERGE   #MULTIPLY   
    Metalliques.opacity = 55.0  #100.0  
    Metalliques.name = "Couleurs_Metalliques "+Metalliquesparms
    
    #pdb.gimp_display_new (newImage)
    #return
    
    GrainsOfColours = pdb.gimp_layer_new_from_visible (newImage, newImage, "")
    newImage.add_layer (GrainsOfColours, 0)
    GrainsOfColours.mode = LAYER_MODE_BURN #SCREEN
    GrainsOfColours.opacity = 100.0
    GrainsOfColours.name = "Grains of colours"
    Mask = pdb.gimp_layer_create_mask (GrainsOfColours, ADD_MASK_COPY)
    pdb.gimp_layer_add_mask (GrainsOfColours, Mask)
    pdb.plug_in_vinvert (newImage, Mask)
    
    # preliminary outcome, for adjustments
    PreOutcome =  pdb.gimp_layer_new_from_visible (newImage, newImage, "")
    newImage.add_layer (PreOutcome, 0)
    PreOutcome.name = "Preliminary Outcome for user adjustments"
    PreOutcome.mode = LAYER_MODE_NORMAL # std
    if AdjLayerMode==1: PreOutcome.mode = LAYER_MODE_VIVID_LIGHT
    if AdjLayerMode==2: PreOutcome.mode = LAYER_MODE_BURN
    if AdjLayerMode==3: PreOutcome.mode = LAYER_MODE_LINEAR_LIGHT
    if AdjLayerMode==4: PreOutcome.mode = LAYER_MODE_MULTIPLY
    if AdjLayerMode==5: PreOutcome.mode = LAYER_MODE_LINEAR_BURN
    
    # optional Source on Top in Dissolve mode
    if SourceOnTop==1:
        SOT = Painted.copy()
        newImage.add_layer (SOT,0)
        SOT.mode = LAYER_MODE_DISSOLVE
        SOT.opacity = 50.0
        SOT.name = "Painted on Top"
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" with grains of colours (fake Pointillisme look) "
    effectLs = ["basic", "regular", "fine" ]
    effectLused = " (effect look "+effectLs[EffectLook]+")"
    outcome.name = outcome.name+effectLused
    ALMODES=["NORMAL","VIVID_LIGHT","BURN","LINEAR_LIGHT","MULTIPLY","LINEAR_BURN"]
    ALMODEused = " (adj layer mode "+ALMODES[AdjLayerMode]+")"
    outcome.name = outcome.name+ALMODEused
    if SourceOnTop==1:
        outcome.name = outcome.name+" (Source on Top)"
        
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != WrkWH:
        uRatio = UserOutWH*1.00/WrkWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
    
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2022"
desc="Grains Of Colours (fake Pointillisme)"

register(
    "GrainsOfColours",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_OPTION, "effect", "Effect look", 1, ["Basic", "Regular", "Fine" ]),
        (PF_OPTION, "adjust", "Adjusting layer mode"+
                               "\n (for other modes set Normal then change)",
                               1, ["Normal", "Vivid Light", "Burn",
                                   "Linear Light", "Multiply", "Linear Burn"]),
        (PF_OPTION, "source", "Source on Top",
                               1, ["No", 
                                   "Yes (Dissolve)"]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    GrainsOfColours,
    menu=menu
)

main()
